﻿namespace HeuristicLab.BenchmarkSuite.Problems {
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Boolean;

  public class MirrorImage : BenchmarkSuiteDataDescriptor {
    private const string name = "Mirror Image - Easy";
    private const string fileName = "MirrorImage.csv";
    private const string description = "Given two vectors of integers, return true if one vector is the reverse of the other, and false otherwise.";

    protected override string FileName { get { return fileName; } }
    public override string Name { get { return name; } }
    public override string Description { get { return description; } }
    protected override int InputArgumentCount { get { return 2; } }
    protected override int OutputArgumentCount { get { return 1; } }

    public override ProblemData CreateProblemData() {
      return new ProblemData(ProblemType.MirrorImage) {
        Name = Name,
        Description = Description,
        Examples = CloneExamples(),
        BestResult = 0,
        WorstResult = 1,
        InputArgumentTypes = new[] { ExampleArgumentType.IntegerVector, ExampleArgumentType.IntegerVector },
        OutputArgumentTypes = new[] { ExampleArgumentType.Boolean },
        TrainingCount = 100,
        TestCount = 1000,
        EnabledDataTypes = DataTypes.Exec | DataTypes.Integer | DataTypes.Boolean | DataTypes.IntegerVector,
        MaxSize = 300,
        EvalLimit = 600,
        ErcOptions = {
          ErcProbability = 0.05,
          BooleanErcOptions = new BooleanErcOptions(
            new BooleanRandomErc {
              AllowFalse = true,
              AllowTrue = true
            })
        }
      };
    }

    protected override Example ParseExample(string[] input, string[] output) {
      return new Example {
        InputArgs = input,
        OutputArgs = output,
        InputIntegerVector = new[] {
          ExampleArgumentConverter.ConvertIntegers(input[0]),
          ExampleArgumentConverter.ConvertIntegers(input[1])
        },
        OutputBoolean = ExampleArgumentConverter.ConvertBooleans(output[0]),
      };
    }
  }
}
